﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.DirectoryServices.AccountManagement;
using System.Web.Configuration;

namespace WinntSecurityProviders
{
    public class GroupConfigElement : ConfigurationElement
    {
        public GroupConfigElement(String newRoleName, String newWindowsGroupName, String newIdentityType)
        {
            this.RoleName = newRoleName;
            this.WindowsGroupName = newWindowsGroupName;
            this.IdentityType = newIdentityType;
        }

        public GroupConfigElement()
        {
        }

        [ConfigurationProperty("WindowsGroupName",
            DefaultValue = @"localhost\nobody",
            IsRequired = true,
            IsKey = true)]
        public string WindowsGroupName
        {
            get
            {
                return (string)this["WindowsGroupName"];
            }
            set
            {
                this["WindowsGroupName"] = value;
            }
        }

        [ConfigurationProperty("IdentityType",
            DefaultValue = @"SamAccountName",
            IsRequired = false,
            IsKey = false)]
        public string IdentityType
        {
            get
            {
                return (string)this["IdentityType"];
            }
            set
            {
                this["IdentityType"] = value;
            }
        }

        [ConfigurationProperty("RoleName",
            DefaultValue = "Undefined",
            IsRequired = true)]
        public string RoleName
        {
            get
            {
                return (string)this["RoleName"];
            }
            set
            {
                this["RoleName"] = value;
            }
        }

        protected override void DeserializeElement(
           System.Xml.XmlReader reader,
            bool serializeCollectionKey)
        {
            base.DeserializeElement(reader,
                serializeCollectionKey);
        }

        protected override bool SerializeElement(
            System.Xml.XmlWriter writer,
            bool serializeCollectionKey)
        {
            bool ret = base.SerializeElement(writer,
                serializeCollectionKey);
            return ret;

        }

        protected override bool IsModified()
        {
            bool ret = base.IsModified();
            return ret;
        }
    }

    public class GroupCollection : ConfigurationElementCollection
    {
        public GroupCollection()
        {
            GroupConfigElement url = (GroupConfigElement)CreateNewElement();
            Add(url);
        }

        public override ConfigurationElementCollectionType CollectionType
        {
            get
            {
                return ConfigurationElementCollectionType.AddRemoveClearMap;
            }
        }

        protected override ConfigurationElement CreateNewElement()
        {
            return new GroupConfigElement();
        }


        protected ConfigurationElement CreateNewElement(String RoleName, String WindowsGroupName, String IdentityType)
        {
            return new GroupConfigElement(RoleName, WindowsGroupName, IdentityType);
        }


        protected override Object GetElementKey(ConfigurationElement element)
        {
            return ((GroupConfigElement)element).WindowsGroupName;
        }


        public new string AddElementName
        {
            get
            {
                return base.AddElementName;
            }

            set
            {
                base.AddElementName = value;
            }

        }

        public new string ClearElementName
        {
            get
            {
                return base.ClearElementName;
            }

            set
            {
                base.ClearElementName = value;
            }

        }

        public new string RemoveElementName
        {
            get
            {
                return base.RemoveElementName;
            }
        }

        public new int Count
        {
            get
            {
                return base.Count;
            }
        }


        public GroupConfigElement this[int index]
        {
            get
            {
                return (GroupConfigElement)BaseGet(index);
            }
            set
            {
                if (BaseGet(index) != null)
                {
                    BaseRemoveAt(index);
                }
                BaseAdd(index, value);
            }
        }

        new public GroupConfigElement this[string Name]
        {
            get
            {
                return (GroupConfigElement)BaseGet(Name);
            }
        }

        public int IndexOf(GroupConfigElement group)
        {
            return BaseIndexOf(group);
        }

        public void Add(GroupConfigElement group)
        {
            BaseAdd(group);
        }

        protected override void BaseAdd(ConfigurationElement element)
        {
            BaseAdd(element, false);
        }

        public void Remove(GroupConfigElement group)
        {
            if (BaseIndexOf(group) >= 0)
                BaseRemove(group.WindowsGroupName);
        }

        public void RemoveAt(int index)
        {
            BaseRemoveAt(index);
        }

        public void Remove(string name)
        {
            BaseRemove(name);
        }

        public void Clear()
        {
            BaseClear();
        }
    }

    /// <summary>
    /// Add following section definition to the <configSections> element in web.config:
    ///     <section name="RoleGroups" type="WinntSecurityProviders.RoleWindowsGroupSection, WinntSecurityProviders" />
    /// Add the following section to the <configuration> element
    /// <RoleGroups>
    ///     <RoleWindowsGroups>
    ///         <clear />
    ///         <add WindowsGroupName="domain\app_admins" RoleName="Application administrator">
    ///     </RoleWindowsGroups>
    /// </RoleGroups>
    /// </summary>
    public class RoleWindowsGroupSection : ConfigurationSection
    {
        public RoleWindowsGroupSection()
        {
        }

        [ConfigurationProperty("RoleWindowsGroups", IsDefaultCollection = true)]
        public GroupCollection Groups
        {
            get
            {
                GroupCollection GroupCollection = (GroupCollection)base["RoleWindowsGroups"];
                return GroupCollection;
            }
        }

        protected override void DeserializeSection(
            System.Xml.XmlReader reader)
        {
            base.DeserializeSection(reader);
        }

        protected override string SerializeSection(
            ConfigurationElement parentElement,
            string name, ConfigurationSaveMode saveMode)
        {
            string s = base.SerializeSection(parentElement, name, saveMode);
            return s;
        }
    }

    public sealed class RoleConfigurationHelper
    {
        public class RoleMapping
        {
            public string RoleName { get; private set; }
            public string WindowsGroupName { get; private set; }
            public IdentityType IdentityType { get; private set; }
            
            public RoleMapping(string RoleName, string WindowsGroupName, string IdentityType)
            {
                this.RoleName = RoleName;
                this.WindowsGroupName = WindowsGroupName;
                this.IdentityType = (IdentityType)Enum.Parse(typeof(IdentityType), IdentityType);
            }
        }

        private static IList<RoleMapping> RoleGroupsCache = new List<RoleMapping>();

        /// <summary>
        /// Function will retrieve mappings from the current site configuration file.
        /// Please note, that values are cahced on first processing. You will need to restart the application for changes to become effective.
        /// </summary>
        /// <returns>An enumeration of the role mappings </returns>
        public static IEnumerable<RoleMapping> GetRoleGroups()
        {
            if (RoleGroupsCache.Count == 0)
            {
                try
                {
                    var sections = WebConfigurationManager.OpenWebConfiguration("/");
                    foreach (ConfigurationSection section in sections.Sections)
                    {
                        if (section is RoleWindowsGroupSection)
                        {
                            IList<RoleMapping> RoleGroups = new List<RoleMapping>();

                            foreach (GroupConfigElement RoleGroup in (section as RoleWindowsGroupSection).Groups)
                            {
                                RoleGroups.Add(new RoleMapping(RoleGroup.RoleName, RoleGroup.WindowsGroupName, RoleGroup.IdentityType));
                            }

                            RoleGroupsCache = RoleGroups;
                        }
                    }
                }
                catch (Exception ex)
                {
                    throw new ConfigurationErrorsException("Failed to load RoleWindowsGroupSection section", ex);
                }
            }

            return RoleGroupsCache;
        }
    }
}
