﻿using System;
using System.DirectoryServices;
using System.DirectoryServices.ActiveDirectory;

namespace WinntSecurityProviders
{
    static class SecurityProviderHelpers
    {
        public enum AuthenticationScope
        {
            Machine,
            Domain
        }
        
        public sealed class DomainUser
        {
            public string PrincipalName { get; private set; }
            public string DomainMame { get; private set; }

            public DomainUser(string UserNameWithDomain)
            {
                string[] dn = UserNameWithDomain.Split('\\');
                this.PrincipalName = dn[1].ToUpper();
                this.DomainMame = dn[0].ToUpper();
            }
        }
        
        internal static bool Authenticate(string userName, string password, string domain)
        {
            bool authentic = false;
            try
            {
                DirectoryEntry entry = new DirectoryEntry("LDAP://" + domain, userName, password);
                authentic = true;
            }
            catch (DirectoryServicesCOMException) { }
            return authentic;
        }

        public static string FriendlyDomainToLdapDomain(string friendlyDomainName)
        {
            string ldapPath = null;
            try
            {
                DirectoryContext objContext = new DirectoryContext(DirectoryContextType.Domain, friendlyDomainName);
                Domain objDomain = Domain.GetDomain(objContext);
                ldapPath = objDomain.Name;
            }
            catch (DirectoryServicesCOMException e)
            {
                ldapPath = e.Message.ToString();
            }
            return ldapPath;
        }

        public static AuthenticationScope ToAuthenticationScope(string scope)
        {
            foreach (var value in Enum.GetValues(typeof(AuthenticationScope)))
            {
                if (((AuthenticationScope)value).ToString() == scope) return (AuthenticationScope)value;
            }

            throw new ArgumentOutOfRangeException("scope", String.Format("Value '{0}' is not among the accepted scopes: ", scope, String.Join(", ", Enum.GetNames(typeof(AuthenticationScope)))));
        }
    }
}
