﻿using System;
using System.DirectoryServices.AccountManagement;
using System.Runtime.InteropServices;
using System.Web.Security;

namespace WinntSecurityProviders
{
    public class WinntMembershipUser : MembershipUser
    {
        public string DisplayName { get; set; }

        public WinntMembershipUser(string providername,
                                  string username,
                                  object providerUserKey,
                                  string email,
                                  string passwordQuestion,
                                  string comment,
                                  bool isApproved,
                                  bool isLockedOut,
                                  DateTime creationDate,
                                  DateTime lastLoginDate,
                                  DateTime lastActivityDate,
                                  DateTime lastPasswordChangedDate,
                                  DateTime lastLockedOutDate,
                                  string DisplayName) :
                                  base(providername,
                                       username,
                                       providerUserKey,
                                       email,
                                       passwordQuestion,
                                       comment,
                                       isApproved,
                                       isLockedOut,
                                       creationDate,
                                       lastLoginDate,
                                       lastActivityDate,
                                       lastPasswordChangedDate,
                                       lastLockedOutDate)
        {
            this.DisplayName = DisplayName;
        }
    }
    
    public class WindowsMembershipProvider : MembershipProvider
    {
        #region P/Invoke part fow WinNT authentication

        [DllImport("advapi32.dll", SetLastError = true)]
        static extern bool LogonUser(
          string principal,
          string authority,
          string password,
          LogonSessionType logonType,
          LogonProvider logonProvider,
          out IntPtr token);

        enum LogonSessionType : uint
        {
            Interactive = 2,
            Network,
            Batch,
            Service,
            NetworkCleartext = 8,
            NewCredentials
        }
        
        enum LogonProvider : uint
        {
            Default = 0, // default for platform (use this!)
            WinNT35,     // sends smoke signals to authority
            WinNT40,     // uses NTLM
            WinNT50      // negotiates Kerb or NTLM
        }
        #endregion

        private SecurityProviderHelpers.AuthenticationScope scope;

        public override void Initialize(string name, System.Collections.Specialized.NameValueCollection config)
        {
            scope = SecurityProviderHelpers.ToAuthenticationScope(config["scope"]);

            base.Initialize(name, config);
        }
        
        public override bool ValidateUser(string username, string password)
        {            
            SecurityProviderHelpers.DomainUser dn = new SecurityProviderHelpers.DomainUser(username);
            if (scope == SecurityProviderHelpers.AuthenticationScope.Machine)
            {
                IntPtr token = IntPtr.Zero;
                return LogonUser(dn.PrincipalName, dn.DomainMame, password, LogonSessionType.Network, LogonProvider.Default, out token);
            }
            else
            {
                return SecurityProviderHelpers.Authenticate(dn.PrincipalName, password, dn.DomainMame);
            }
        }       

        public override MembershipUser GetUser(string username, bool userIsOnline)
        {
            SecurityProviderHelpers.DomainUser dn = new SecurityProviderHelpers.DomainUser(username);
            PrincipalContext ctx;

            if (scope == SecurityProviderHelpers.AuthenticationScope.Domain)
            {
                string domainName = SecurityProviderHelpers.FriendlyDomainToLdapDomain(dn.DomainMame);
                ctx = new PrincipalContext(ContextType.Domain, domainName);
            }
            else
            {
                ctx = new PrincipalContext(ContextType.Machine, dn.DomainMame);
            }

            UserPrincipal user = UserPrincipal.FindByIdentity(ctx, dn.PrincipalName);

            return new WinntMembershipUser(
                this.GetType().Name,
                user.Name,
                string.Join(@"\", ctx.ConnectedServer, user.SamAccountName),
                user.EmailAddress,
                string.Empty,
                user.Description,
                user.Enabled ?? true,
                user.IsAccountLockedOut(),
                DateTime.MinValue,
                user.LastLogon ?? DateTime.MinValue,
                user.LastLogon ?? DateTime.MinValue,
                user.LastPasswordSet ?? DateTime.MinValue,
                user.AccountLockoutTime ?? DateTime.MinValue,
                user.DisplayName
                );
        }

        public override MembershipUser GetUser(object providerUserKey, bool userIsOnline)
        {
            SecurityProviderHelpers.DomainUser dn = new SecurityProviderHelpers.DomainUser(providerUserKey.ToString());
            PrincipalContext ctx;

            if (scope == SecurityProviderHelpers.AuthenticationScope.Domain)
            {
                string domainName = SecurityProviderHelpers.FriendlyDomainToLdapDomain(dn.DomainMame);
                ctx = new PrincipalContext(ContextType.Domain, domainName);
            }
            else
            {
                ctx = new PrincipalContext(ContextType.Machine, dn.DomainMame);
            }

            UserPrincipal user = UserPrincipal.FindByIdentity(ctx, dn.PrincipalName);

            return new WinntMembershipUser(                
                this.GetType().Name,
                user.Name,
                string.Join(@"\", ctx.ConnectedServer, user.SamAccountName),
                user.EmailAddress,
                string.Empty,
                user.Description,
                user.Enabled ?? true,
                user.IsAccountLockedOut(),
                DateTime.MinValue,
                user.LastLogon ?? DateTime.MinValue,
                user.LastLogon ?? DateTime.MinValue,
                user.LastPasswordSet ?? DateTime.MinValue,
                user.AccountLockoutTime ?? DateTime.MinValue,
                user.DisplayName
                );
        }

        #region Not implemented methods and properties
        public void CreateUser(string username, string fullname, string email)
        {
            throw new NotImplementedException();
        }

        public override string ApplicationName
        {
            get
            {
                throw new NotImplementedException();
            }
            set
            {
                throw new NotImplementedException();
            }
        }

        public override bool ChangePassword(string username, string oldPassword, string newPassword)
        {
            throw new NotImplementedException();
        }

        public override bool ChangePasswordQuestionAndAnswer(string username, string password, string newPasswordQuestion, string newPasswordAnswer)
        {
            throw new NotImplementedException();
        }        

        public override bool DeleteUser(string username, bool deleteAllRelatedData)
        {
            throw new NotImplementedException();
        }

        public override bool EnablePasswordReset
        {
            get { throw new NotImplementedException(); }
        }

        public override bool EnablePasswordRetrieval
        {
            get { throw new NotImplementedException(); }
        }

        public override MembershipUserCollection FindUsersByEmail(string emailToMatch, int pageIndex, int pageSize, out int totalRecords)
        {
            throw new NotImplementedException();
        }

        public override MembershipUserCollection FindUsersByName(string usernameToMatch, int pageIndex, int pageSize, out int totalRecords)
        {
            throw new NotImplementedException();
        }

        public override MembershipUserCollection GetAllUsers(int pageIndex, int pageSize, out int totalRecords)
        {
            throw new NotImplementedException();
        }

        public override int GetNumberOfUsersOnline()
        {
            throw new NotImplementedException();
        }

        public override string GetPassword(string username, string answer)
        {
            throw new NotImplementedException();
        }        

        public override string GetUserNameByEmail(string email)
        {
            throw new NotImplementedException();
        }

        public override int MaxInvalidPasswordAttempts
        {
            get { throw new NotImplementedException(); }
        }

        public override int MinRequiredNonAlphanumericCharacters
        {
            get { throw new NotImplementedException(); }
        }

        public override int MinRequiredPasswordLength
        {
            get { throw new NotImplementedException(); }
        }

        public override int PasswordAttemptWindow
        {
            get { throw new NotImplementedException(); }
        }

        public override MembershipPasswordFormat PasswordFormat
        {
            get { throw new NotImplementedException(); }
        }

        public override string PasswordStrengthRegularExpression
        {
            get { throw new NotImplementedException(); }
        }

        public override bool RequiresQuestionAndAnswer
        {
            get { throw new NotImplementedException(); }
        }

        public override bool RequiresUniqueEmail
        {
            get { throw new NotImplementedException(); }
        }

        public override string ResetPassword(string username, string answer)
        {
            throw new NotImplementedException();
        }

        public override bool UnlockUser(string userName)
        {
            throw new NotImplementedException();
        }

        public override void UpdateUser(MembershipUser user)
        {
            throw new NotImplementedException();
        }

        public override MembershipUser CreateUser(string username, string password, string email, string passwordQuestion, string passwordAnswer, bool isApproved, object providerUserKey, out MembershipCreateStatus status)
        {
            throw new NotImplementedException();
        }
        #endregion
    }
}
